# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .folder_entity_test_suite_data import FolderEntityTestSuiteData
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class FolderEntityTestSuite(UniversalBaseModel):
    """
    A slim representation of a Test Suite, as it exists within a Folder.
    """

    id: str
    type: typing.Literal["TEST_SUITE"] = "TEST_SUITE"
    data: FolderEntityTestSuiteData

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
