# This file was auto-generated by Fern from our API Definition.

import typing
from .test_suite_run_execution_string_output import TestSuiteRunExecutionStringOutput
from .test_suite_run_execution_number_output import TestSuiteRunExecutionNumberOutput
from .test_suite_run_execution_json_output import TestSuiteRunExecutionJsonOutput
from .test_suite_run_execution_chat_history_output import TestSuiteRunExecutionChatHistoryOutput
from .test_suite_run_execution_search_results_output import TestSuiteRunExecutionSearchResultsOutput
from .test_suite_run_execution_error_output import TestSuiteRunExecutionErrorOutput
from .test_suite_run_execution_function_call_output import TestSuiteRunExecutionFunctionCallOutput
from .test_suite_run_execution_array_output import TestSuiteRunExecutionArrayOutput

TestSuiteRunExecutionOutput = typing.Union[
    TestSuiteRunExecutionStringOutput,
    TestSuiteRunExecutionNumberOutput,
    TestSuiteRunExecutionJsonOutput,
    TestSuiteRunExecutionChatHistoryOutput,
    TestSuiteRunExecutionSearchResultsOutput,
    TestSuiteRunExecutionErrorOutput,
    TestSuiteRunExecutionFunctionCallOutput,
    TestSuiteRunExecutionArrayOutput,
]
