# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .array_vellum_value import ArrayVellumValue
import datetime as dt
import pydantic
import typing
from .vellum_variable import VellumVariable
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeploymentHistoryItem(UniversalBaseModel):
    id: str
    deployment_id: str
    timestamp: dt.datetime
    label: str = pydantic.Field()
    """
    A human-readable label for the deployment
    """

    name: str = pydantic.Field()
    """
    A name that uniquely identifies this deployment within its workspace
    """

    input_variables: typing.List[VellumVariable]
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable description of the deployment
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
