# This file was auto-generated by Fern from our API Definition.

import typing
from .terminal_node_string_result import TerminalNodeStringResult
from .terminal_node_number_result import TerminalNodeNumberResult
from .terminal_node_json_result import TerminalNodeJsonResult
from .terminal_node_chat_history_result import TerminalNodeChatHistoryResult
from .terminal_node_search_results_result import TerminalNodeSearchResultsResult
from .terminal_node_error_result import TerminalNodeErrorResult
from .terminal_node_array_result import TerminalNodeArrayResult
from .terminal_node_function_call_result import TerminalNodeFunctionCallResult

TerminalNodeResultOutput = typing.Union[
    TerminalNodeStringResult,
    TerminalNodeNumberResult,
    TerminalNodeJsonResult,
    TerminalNodeChatHistoryResult,
    TerminalNodeSearchResultsResult,
    TerminalNodeErrorResult,
    TerminalNodeArrayResult,
    TerminalNodeFunctionCallResult,
]
