# This file was auto-generated by Fern from our API Definition.

import typing
from .test_suite_test_case_create_bulk_operation_request import TestSuiteTestCaseCreateBulkOperationRequest
from .test_suite_test_case_replace_bulk_operation_request import TestSuiteTestCaseReplaceBulkOperationRequest
from .test_suite_test_case_upsert_bulk_operation_request import TestSuiteTestCaseUpsertBulkOperationRequest
from .test_suite_test_case_delete_bulk_operation_request import TestSuiteTestCaseDeleteBulkOperationRequest

TestSuiteTestCaseBulkOperationRequest = typing.Union[
    TestSuiteTestCaseCreateBulkOperationRequest,
    TestSuiteTestCaseReplaceBulkOperationRequest,
    TestSuiteTestCaseUpsertBulkOperationRequest,
    TestSuiteTestCaseDeleteBulkOperationRequest,
]
