# This file was auto-generated by Fern from our API Definition.

import typing
from .node_execution_initiated_event import NodeExecutionInitiatedEvent
from .node_execution_streaming_event import NodeExecutionStreamingEvent
from .node_execution_fulfilled_event import NodeExecutionFulfilledEvent
from .node_execution_rejected_event import NodeExecutionRejectedEvent
from .node_execution_paused_event import NodeExecutionPausedEvent
from .node_execution_resumed_event import NodeExecutionResumedEvent

VellumNodeExecutionEvent = typing.Union[
    NodeExecutionInitiatedEvent,
    NodeExecutionStreamingEvent,
    NodeExecutionFulfilledEvent,
    NodeExecutionRejectedEvent,
    NodeExecutionPausedEvent,
    NodeExecutionResumedEvent,
]
