# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .array_vellum_value import ArrayVellumValue
from .chat_message_prompt_block import ChatMessagePromptBlock
import typing
from .vellum_variable import VellumVariable
from .prompt_parameters import PromptParameters
from .prompt_settings import PromptSettings
from .prompt_block import PromptBlock
from .function_definition import FunctionDefinition
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PromptExecConfig(UniversalBaseModel):
    ml_model: str
    input_variables: typing.List[VellumVariable]
    parameters: PromptParameters
    settings: typing.Optional[PromptSettings] = None
    blocks: typing.List[PromptBlock]
    functions: typing.Optional[typing.List[FunctionDefinition]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
