# This file was auto-generated by Fern from our API Definition.

import typing
from .workflow_execution_actual_string_request import WorkflowExecutionActualStringRequest
from .workflow_execution_actual_json_request import WorkflowExecutionActualJsonRequest
from .workflow_execution_actual_chat_history_request import WorkflowExecutionActualChatHistoryRequest

SubmitWorkflowExecutionActualRequest = typing.Union[
    WorkflowExecutionActualStringRequest, WorkflowExecutionActualJsonRequest, WorkflowExecutionActualChatHistoryRequest
]
