# This file was auto-generated by Fern from our API Definition.

from .prompt_version_build_config_sandbox import PromptVersionBuildConfigSandbox

ComponentsSchemasPromptVersionBuildConfigSandbox = PromptVersionBuildConfigSandbox
