# This file was auto-generated by Fern from our API Definition.

import typing
from .prompt_request_string_input import PromptRequestStringInput
from .prompt_request_json_input import PromptRequestJsonInput
from .prompt_request_chat_history_input import PromptRequestChatHistoryInput

PromptRequestInput = typing.Union[PromptRequestStringInput, PromptRequestJsonInput, PromptRequestChatHistoryInput]
