# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TestSuiteRunDeploymentReleaseTagExecConfigDataRequest(UniversalBaseModel):
    deployment_id: str = pydantic.Field()
    """
    The ID of the Prompt Deployment to run the Test Suite against.
    """

    tag: typing.Optional[str] = pydantic.Field(default=None)
    """
    A tag identifying which release of the Prompt Deployment to run the Test Suite against. Useful for testing past versions of the Prompt Deployment
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
