# This file was auto-generated by Fern from our API Definition.

import typing
from .workflow_execution_span import WorkflowExecutionSpan
from .node_execution_span import NodeExecutionSpan

VellumSpan = typing.Union[WorkflowExecutionSpan, NodeExecutionSpan]
