# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .components_schemas_prompt_version_build_config_sandbox import ComponentsSchemasPromptVersionBuildConfigSandbox
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class PromptDeploymentReleasePromptVersion(UniversalBaseModel):
    id: str
    build_config: ComponentsSchemasPromptVersionBuildConfigSandbox = pydantic.Field()
    """
    Configuration used to build this prompt version.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
