# This file was auto-generated by Fern from our API Definition.

import typing
from .workflow_execution_initiated_event import WorkflowExecutionInitiatedEvent
from .workflow_execution_streaming_event import WorkflowExecutionStreamingEvent
from .workflow_execution_fulfilled_event import WorkflowExecutionFulfilledEvent
from .workflow_execution_rejected_event import WorkflowExecutionRejectedEvent
from .workflow_execution_paused_event import WorkflowExecutionPausedEvent
from .workflow_execution_resumed_event import WorkflowExecutionResumedEvent
from .workflow_execution_snapshotted_event import WorkflowExecutionSnapshottedEvent

VellumWorkflowExecutionEvent = typing.Union[
    WorkflowExecutionInitiatedEvent,
    WorkflowExecutionStreamingEvent,
    WorkflowExecutionFulfilledEvent,
    WorkflowExecutionRejectedEvent,
    WorkflowExecutionPausedEvent,
    WorkflowExecutionResumedEvent,
    WorkflowExecutionSnapshottedEvent,
]
