# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class CompilePromptMeta(UniversalBaseModel):
    """
    The subset of the metadata tracked by Vellum during Prompt Deployment compilation that the request opted into with `expand_meta`.
    """

    model_name: typing.Optional[str] = None
    deployment_release_tag: typing.Optional[str] = None
    prompt_version_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
