# This file was auto-generated by Fern from our API Definition.

import typing
from .string_input import StringInput
from .json_input import JsonInput
from .chat_history_input import ChatHistoryInput
from .number_input import NumberInput
from .search_results_input import SearchResultsInput
from .error_input import ErrorInput
from .array_input import ArrayInput
from .function_call_input import FunctionCallInput
from .code_executor_secret_input import CodeExecutorSecretInput

CodeExecutorInput = typing.Union[
    StringInput,
    JsonInput,
    ChatHistoryInput,
    NumberInput,
    SearchResultsInput,
    ErrorInput,
    ArrayInput,
    FunctionCallInput,
    CodeExecutorSecretInput,
]
