# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import datetime as dt
from .new_member_join_behavior_enum import NewMemberJoinBehaviorEnum
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class OrganizationRead(UniversalBaseModel):
    id: str
    name: str
    created: typing.Optional[dt.datetime] = None
    allow_staff_access: typing.Optional[bool] = None
    new_member_join_behavior: NewMemberJoinBehaviorEnum
    limit_config: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
