# This file was auto-generated by Fern from our API Definition.

import typing
from .templating_node_string_result import TemplatingNodeStringResult
from .templating_node_number_result import TemplatingNodeNumberResult
from .templating_node_json_result import TemplatingNodeJsonResult
from .templating_node_chat_history_result import TemplatingNodeChatHistoryResult
from .templating_node_search_results_result import TemplatingNodeSearchResultsResult
from .templating_node_error_result import TemplatingNodeErrorResult
from .templating_node_array_result import TemplatingNodeArrayResult
from .templating_node_function_call_result import TemplatingNodeFunctionCallResult

TemplatingNodeResultOutput = typing.Union[
    TemplatingNodeStringResult,
    TemplatingNodeNumberResult,
    TemplatingNodeJsonResult,
    TemplatingNodeChatHistoryResult,
    TemplatingNodeSearchResultsResult,
    TemplatingNodeErrorResult,
    TemplatingNodeArrayResult,
    TemplatingNodeFunctionCallResult,
]
