# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing
from .vellum_value_logical_condition_request import VellumValueLogicalConditionRequest
import typing

if typing.TYPE_CHECKING:
    from .vellum_value_logical_condition_group_request import VellumValueLogicalConditionGroupRequest
VellumValueLogicalExpressionRequest = typing.Union[
    VellumValueLogicalConditionRequest, "VellumValueLogicalConditionGroupRequest"
]
