# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
import typing
from .string_vellum_value import StringVellumValue
from .number_vellum_value import NumberVellumValue
from .json_vellum_value import JsonVellumValue
from .audio_vellum_value import AudioVellumValue
from .video_vellum_value import VideoVellumValue
from .image_vellum_value import ImageVellumValue
from .document_vellum_value import DocumentVellumValue
from .function_call_vellum_value import FunctionCallVellumValue
from .error_vellum_value import ErrorVellumValue
from .chat_history_vellum_value import ChatHistoryVellumValue
from .search_results_vellum_value import SearchResultsVellumValue
from .thinking_vellum_value import ThinkingVellumValue
import typing

if typing.TYPE_CHECKING:
    from .array_vellum_value import ArrayVellumValue
VellumValue = typing.Union[
    StringVellumValue,
    NumberVellumValue,
    JsonVellumValue,
    AudioVellumValue,
    VideoVellumValue,
    ImageVellumValue,
    DocumentVellumValue,
    FunctionCallVellumValue,
    ErrorVellumValue,
    "ArrayVellumValue",
    ChatHistoryVellumValue,
    SearchResultsVellumValue,
    ThinkingVellumValue,
]
