# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .token_overlapping_window_chunker_config_request import TokenOverlappingWindowChunkerConfigRequest
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class TokenOverlappingWindowChunkingRequest(UniversalBaseModel):
    """
    Token overlapping window chunking
    """

    chunker_name: typing.Literal["token-overlapping-window-chunker"] = "token-overlapping-window-chunker"
    chunker_config: typing.Optional[TokenOverlappingWindowChunkerConfigRequest] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
