# This file was auto-generated by Fern from our API Definition.

import typing
from .initiated_workflow_node_result_event import InitiatedWorkflowNodeResultEvent
from .streaming_workflow_node_result_event import StreamingWorkflowNodeResultEvent
from .fulfilled_workflow_node_result_event import FulfilledWorkflowNodeResultEvent
from .rejected_workflow_node_result_event import RejectedWorkflowNodeResultEvent

WorkflowNodeResultEvent = typing.Union[
    InitiatedWorkflowNodeResultEvent,
    StreamingWorkflowNodeResultEvent,
    FulfilledWorkflowNodeResultEvent,
    RejectedWorkflowNodeResultEvent,
]
