# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .vellum_code_resource_definition import VellumCodeResourceDefinition
import typing
from .external_input_descriptor import ExternalInputDescriptor
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class WorkflowExecutionPausedBody(UniversalBaseModel):
    workflow_definition: VellumCodeResourceDefinition
    external_inputs: typing.List[ExternalInputDescriptor]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
